/*
 * Vocoder
 *  2012 Achim Settelmeier <vocoder@m1.sirlab.de>
 *
 */

// Includes
#include <string>
#include <stdexcept>
#include <math.h>
#include "inputtrack.h"

using std::string;
using std::runtime_error;


// InputTrack
InputTrack::InputTrack(const string &filename,
		       const nframes_t _maxFrames,
		       const nframes_t _fps):
	Track(_maxFrames, _fps),
	stretchFactor(1.0),
	inputBufferSize(0),
	inputBufferPtr(NULL)
{
	sndFileInfo.format=0;
	sndFileFDPtr=sf_open(filename.c_str(), SFM_READ, &sndFileInfo);
	
	if(sndFileFDPtr==NULL)
		throw runtime_error(filename + ": " + sf_strerror(NULL));
	
	if(sndFileInfo.channels>2)
		throw runtime_error(filename + ": too many channels (only mono or stereo supported)");
	
	stretchFactor=double(fps) / sndFileInfo.samplerate;
	inputBufferSize=lround(maxFrames / stretchFactor);
	inputBufferPtr=new sample_t[inputBufferSize * sndFileInfo.channels];
}

// ~InputTrack
InputTrack::~InputTrack(){
	sf_close(sndFileFDPtr);
	
	if(inputBufferPtr)
		delete inputBufferPtr;
}

// readBlock
nframes_t InputTrack::readBlock(const nframes_t numFrames){
	
	nframes_t numFramesToRead=lround(numFrames / stretchFactor);
	nframes_t numFramesRead=sf_readf_float(sndFileFDPtr,
					       inputBufferPtr,
					       numFramesToRead);
	
	nframes_t numFramesAvailable=lround(numFramesRead * stretchFactor);
	// level out rouding errors
	if(numFramesToRead==numFramesRead)
		numFramesAvailable=numFrames;
	
	// resample from inputBufferPtr to sampleBufferPtr 
	for(nframes_t dstFrameNr=0; dstFrameNr<numFrames; dstFrameNr++){
		if(sndFileInfo.channels==1){
			// mono input
			nframes_t srcFrameNr=lround(dstFrameNr / stretchFactor);
			sampleBufferPtr[dstFrameNr * 2]=inputBufferPtr[srcFrameNr];
			sampleBufferPtr[dstFrameNr * 2 + 1]=inputBufferPtr[srcFrameNr];
		}else{
			// stereo input
			nframes_t srcFrameNr=lround(dstFrameNr / stretchFactor);
			sampleBufferPtr[dstFrameNr * 2]=inputBufferPtr[srcFrameNr * 2 ];
			sampleBufferPtr[dstFrameNr * 2 + 1]=inputBufferPtr[srcFrameNr * 2 + 1];
		}
	}
	
	return numFramesAvailable;
}
